@@ -2,9 +2,7 @@
  * Copyright (c) 2005 Apple Inc. All rights reserved.
  * Copyright (c) 2005-2006 Paul Guyot <pguyot@kallisys.net>,
  * All rights reserved.
- * Copyright (c) 2006-2013 The MacPorts Project
- *
- * $Id$
+ * Copyright (c) 2006-2015 The MacPorts Project
  *
  * @APPLE_BSD_LICENSE_HEADER_START@
  *
@@ -44,6 +42,8 @@
 #endif
 
 #include <errno.h>
+#include <fcntl.h>
+#include <inttypes.h>
 #include <pthread.h>
 #include <string.h>
 #include <sys/attr.h>
@@ -136,7 +136,7 @@ static void __darwintrace_sock_destructor(FILE *dtsock) {
 	__darwintrace_close_sock = fileno(dtsock);
 	fclose(dtsock);
 	__darwintrace_close_sock = -1;
-	pthread_setspecific(sock_key, NULL);
+	__darwintrace_sock_set(NULL);
 }
 
 /**
@@ -316,7 +316,7 @@ static void __darwintrace_get_filemap() {
 		free(newfilemap);
 		if (filemap != NULL)
 			break;
-		newfilemap = __send("filemap\t", (uint32_t) strlen("filemap\t"), 1);
+		newfilemap = __send("filemap\t", 8, 1);
 	} while (!CAS(NULL, newfilemap, &filemap));
 
 #if DARWINTRACE_DEBUG && 0
@@ -339,7 +339,7 @@ void __darwintrace_close() {
 		__darwintrace_close_sock = fileno(dtsock);
 		fclose(dtsock);
 		__darwintrace_close_sock = -1;
-		pthread_setspecific(sock_key, NULL);
+		__darwintrace_sock_set(NULL);
 	}
 }
 
@@ -376,6 +376,7 @@ void __darwintrace_setup() {
 
 	if (__darwintrace_sock() == NULL) {
 		int sock;
+		int sockflags;
 		FILE *stream;
 		struct sockaddr_un sun;
 
@@ -391,6 +392,29 @@ void __darwintrace_setup() {
 			abort();
 		}
 
+		/* Set the close-on-exec flag as early as possible after the socket
+		 * creation. On OS X, there is no way to do this race-condition free
+		 * unless you synchronize around creation and fork(2) -- however,
+		 * blocking in this function is not acceptable for darwintrace, because
+		 * it could possibly run in a signal handler, leading to a deadlock.
+		 *
+		 * The close-on-exec flag is needed because we're using a thread-local
+		 * variable to hold a reference to this socket, but multi-threaded
+		 * programs that fork will only clone the thread that calls fork(2),
+		 * which leaves us with no reference to the other sockets (which are
+		 * inherited, because FDs are process-wide). Consequently, this can
+		 * lead to a resource leak.
+		 */
+		if (-1 == (sockflags = fcntl(sock, F_GETFD))) {
+			perror("darwintrace: fcntl(F_GETFD)");
+			abort();
+		}
+		sockflags |= FD_CLOEXEC;
+		if (-1 == fcntl(sock, F_SETFD, sockflags)) {
+			perror("darwintrace: fcntl(F_SETFD, flags | FD_CLOEXEC)");
+			abort();
+		}
+
 		if (strlen(__env_darwintrace_log) > sizeof(sun.sun_path) - 1) {
 			fprintf(stderr, "darwintrace: Can't connect to socket %s: name too long\n", __env_darwintrace_log);
 			abort();
@@ -426,56 +450,14 @@ void __darwintrace_setup() {
  */
 static inline void __darwintrace_log_op(const char *op, const char *path) {
 	uint32_t size;
-	char pathbuf[MAXPATHLEN];
 	char logbuffer[BUFFER_SIZE];
-	const char *realpath;
 
-	do {
-#if defined(__APPLE__) && defined(ATTR_CMN_FULLPATH) && 0
-		if (false && strncmp(path, "/.vol/", 6) == 0) {
-			// path in VOLFS, try to get inode -> name lookup from
-			// getattrlist(2).
-			struct attrlist attrlist;
-			attrlist.bitmapcount = ATTR_BIT_MAP_COUNT;
-			attrlist.reserved = 0;
-			attrlist.commonattr = ATTR_CMN_FULLPATH;
-			attrlist.volattr = 0;
-			attrlist.dirattr = 0;
-			attrlist.fileattr = 0;
-			attrlist.forkattr = 0;
-
-			char attrbuf[sizeof(uint32_t) + sizeof(attrreference_t) + (PATH_MAX)];
-			/*           attrlength         attrref_t for the name     UTF-8 name up to PATH_MAX chars */
-			if (-1 == (getattrlist(path, &attrlist, attrbuf, sizeof(attrbuf), FSOPT_NOFOLLOW))) {
-				perror("darwintrace: getattrlist");
-				// ignore and just print the /.vol/ path
-			} else {
-				attrreference_t *nameAttrRef = (attrreference_t *) (attrbuf + sizeof(uint32_t));
-				strlcpy(pathbuf, ((char *) nameAttrRef) + nameAttrRef->attr_dataoffset, sizeof(pathbuf));
-				realpath = pathbuf;
-				break;
-			}
-		}
-#endif
-
-		if (*path != '/') {
-			if (!getcwd(pathbuf, sizeof(pathbuf))) {
-				perror("darwintrace: getcwd");
-				abort();
-			}
-
-			strlcat(pathbuf, "/", sizeof(pathbuf));
-			strlcat(pathbuf, path, sizeof(pathbuf));
-			realpath = pathbuf;
-			break;
-		}
-
-		/* otherwise, just copy the original path. */
-		realpath = path;
-	} while (0);
-
-	size = snprintf(logbuffer, sizeof(logbuffer), "%s\t%s", op, realpath);
-	__send(logbuffer, size, 0);
+	size = snprintf(logbuffer, sizeof(logbuffer), "%s\t%s", op, path);
+	// Check if the buffer was short. If it was, discard the message silently,
+	// assuming it isn't important enough to error out.
+	if (size < BUFFER_SIZE) {
+		__send(logbuffer, size, 0);
+	}
 }
 
 /**
@@ -505,7 +487,8 @@ static int dependency_check(const char *path) {
 	}
 
 	len = snprintf(buffer, sizeof(buffer), "dep_check\t%s", path);
-	if (len > sizeof(buffer)) {
+	if (len >= sizeof(buffer)) {
+		fprintf(stderr, "darwintrace: truncating buffer length from %" PRIu32 " to %zu.", len, sizeof(buffer) - 1);
 		len = sizeof(buffer) - 1;
 	}
 	p = __send(buffer, len, 1);
@@ -538,21 +521,37 @@ static int dependency_check(const char *path) {
 }
 
 /**
- * Helper function to recieve a number of bytes from the tracelib communication
+ * Helper function to receive a number of bytes from the tracelib communication
  * socket and deal with any errors that might occur.
  *
  * \param[out] buf buffer to hold received data
  * \param[in]  size number of bytes to read from the socket
  */
 static void frecv(void *restrict buf, size_t size) {
-	FILE *stream = __darwintrace_sock();
-	if (1 != fread(buf, size, 1, stream)) {
-		if (ferror(stream)) {
-			perror("darwintrace: fread");
-		} else {
-			fprintf(stderr, "darwintrace: fread: end-of-file\n");
+	/* We cannot safely use fread(3) here, because we're not in control of the
+	 * application's signal handling settings (which means we must assume
+	 * SA_RESTART isn't set) and fread(3) may return short without giving us
+	 * a way to know how many bytes have actually been read, i.e. without a way
+	 * to do the call again. Because of this great API design and
+	 * implementation on OS X, we'll just use read(2) here. */
+	int fd = fileno(__darwintrace_sock());
+	size_t count = 0;
+	while (count < size) {
+		ssize_t res = read(fd, buf + count, size - count);
+		if (res < 0) {
+			if (errno == EINTR) {
+				continue;
+			}
+			perror("darwintrace: read");
+			abort();
 		}
-		abort();
+
+		if (res == 0) {
+			fprintf(stderr, "darwintrace: read: end-of-file\n");
+			abort();
+		}
+
+		count += res;
 	}
 }
 
@@ -564,16 +563,26 @@ static void frecv(void *restrict buf, size_t size) {
  * \param[in] size number of bytes in the buffer
  */
 static void fsend(const void *restrict buf, size_t size) {
-	FILE *stream = __darwintrace_sock();
-	if (1 != fwrite(buf, size, 1, stream)) {
-		if (ferror(stream)) {
-			perror("darwintrace: fwrite");
-		} else {
-			fprintf(stderr, "darwintrace: fwrite: end-of-file\n");
+	/* We cannot safely use fwrite(3) here, because we're not in control of the
+	 * application's signal handling settings (which means we must assume
+	 * SA_RESTART isn't set) and fwrite(3) may return short without giving us
+	 * a way to know how many bytes have actually been written, i.e. without
+	 * a way to do the call again. Because of this great API design and
+	 * implementation on OS X, we'll just use write(2) here. */
+	int fd = fileno(__darwintrace_sock());
+	size_t count = 0;
+	while (count < size) {
+		ssize_t res = write(fd, buf + count, size - count);
+		if (res < 0) {
+			if (errno == EINTR) {
+				continue;
+			}
+			perror("darwintrace: write");
+			abort();
 		}
-		abort();
+
+		count += res;
 	}
-	fflush(stream);
 }
 
 /**
@@ -713,6 +722,7 @@ static inline bool __darwintrace_sandbox_check(const char *path, int flags) {
 bool __darwintrace_is_in_sandbox(const char *path, int flags) {
 #define lstat(x, y) syscall(LSTATSYSNUM, (x), (y))
 #define readlink(x,y,z) syscall(SYS_readlink, (x), (y), (z))
+#define getattrlist(v,w,x,y,z) syscall(SYS_getattrlist, (v), (w), (x), (y), (z))
 	if (!filemap) {
 		return true;
 	}
@@ -739,12 +749,43 @@ bool __darwintrace_is_in_sandbox(const char *path, int flags) {
 	const char *token = NULL;
 	size_t idx;
 	if (*path != '/') {
-		// The path isn't absolute, start by populating pathcomponents with the
-		// current working directory
+		/*
+		 * The path isn't absolute, start by populating pathcomponents with the
+		 * current working directory.
+		 * 
+		 * However, we avoid getcwd(3) if we can and use getattrlist(2) with
+		 * ATTR_CMN_FULLPATH instead, because getcwd(3) will open all parent
+		 * directories, read them, search for the current component using its
+		 * inode obtained from lstat(., .., ../.., etc.) and build the path
+		 * this way, which is inefficient and will also call back into
+		 * darwintrace code.
+		 */
+#		ifdef ATTR_CMN_FULLPATH
+		struct attrlist attrlist;
+		attrlist.bitmapcount = ATTR_BIT_MAP_COUNT;
+		attrlist.reserved = 0;
+		attrlist.commonattr = ATTR_CMN_FULLPATH;
+		attrlist.volattr = 0;
+		attrlist.dirattr = 0;
+		attrlist.fileattr = 0;
+		attrlist.forkattr = 0;
+
+		char attrbuf[sizeof(uint32_t) + sizeof(attrreference_t) + (PATH_MAX + 1)];
+		/*           attrlength         attrref_t for the name     UTF-8 name up to PATH_MAX chars */
+
+		// FIXME This sometimes violates the stack canary
+		if (-1 == (getattrlist(".", &attrlist, attrbuf, sizeof(attrbuf), FSOPT_NOFOLLOW))) {
+			perror("darwintrace: getattrlist");
+			abort();
+		}
+		attrreference_t *nameAttrRef = (attrreference_t *) (attrbuf + sizeof(uint32_t));
+		strlcpy(normPath, ((char *) nameAttrRef) + nameAttrRef->attr_dataoffset, sizeof(normPath));
+#		else /* defined(ATTR_CMN_FULLPATH) */
 		if (getcwd(normPath, sizeof(normPath)) == NULL) {
 			perror("darwintrace: getcwd");
 			abort();
 		}
+#		endif /* defined(ATTR_CMN_FULLPATH) */
 
 		char *writableToken = normPath + 1;
 		while ((idx = strcspn(writableToken, "/")) > 0) {
@@ -821,6 +862,65 @@ bool __darwintrace_is_in_sandbox(const char *path, int flags) {
 		token += idx + 1;
 	}
 
+	// strip off resource forks
+	if (numComponents >= 2 &&
+		strcmp("..namedfork", pathComponents[numComponents - 2].start) == 0 &&
+		strcmp("rsrc", pathComponents[numComponents - 1].start) == 0) {
+		numComponents -= 2;
+	}
+
+#	ifdef ATTR_CMN_FULLPATH
+	if (numComponents >= 3 && strncmp(".vol", pathComponents[0].start, pathComponents[0].len) == 0) {
+		// path in VOLFS, try to get inode -> name lookup from getattrlist(2).
+
+		// Add the slashes and the terminating \0
+		for (size_t i = 0; i < numComponents; ++i) {
+			if (i == numComponents - 1) {
+				pathComponents[i].start[pathComponents[i].len] = '\0';
+			} else {
+				pathComponents[i].start[pathComponents[i].len] = '/';
+			}
+		}
+
+		struct attrlist attrlist;
+		attrlist.bitmapcount = ATTR_BIT_MAP_COUNT;
+		attrlist.reserved = 0;
+		attrlist.commonattr = ATTR_CMN_FULLPATH;
+		attrlist.volattr = 0;
+		attrlist.dirattr = 0;
+		attrlist.fileattr = 0;
+		attrlist.forkattr = 0;
+
+		char attrbuf[sizeof(uint32_t) + sizeof(attrreference_t) + (PATH_MAX + 1)];
+		/*           attrlength         attrref_t for the name     UTF-8 name up to PATH_MAX chars */
+
+		if (-1 == (getattrlist(normPath, &attrlist, attrbuf, sizeof(attrbuf), FSOPT_NOFOLLOW))) {
+			perror("darwintrace: getattrlist");
+			// ignore and just return the /.vol/ path
+		} else {
+			attrreference_t *nameAttrRef = (attrreference_t *) (attrbuf + sizeof(uint32_t));
+			strlcpy(normPath, ((char *) nameAttrRef) + nameAttrRef->attr_dataoffset, sizeof(normPath));
+
+			numComponents = 0;
+			char *writableToken = normPath + 1;
+			while ((idx = strcspn(writableToken, "/")) > 0) {
+				// found a token, tokenize and store it
+				pathComponents[numComponents].start = writableToken;
+				pathComponents[numComponents].len   = idx;
+				numComponents++;
+
+				bool final = writableToken[idx] == '\0';
+				writableToken[idx] = '\0';
+				if (final) {
+					break;
+				}
+				// advance token
+				writableToken += idx + 1;
+			}
+		}
+	}
+#	endif
+
 	bool pathIsSymlink;
 	size_t loopCount = 0;
 	do {
@@ -931,6 +1031,7 @@ bool __darwintrace_is_in_sandbox(const char *path, int flags) {
 	} while (pathIsSymlink);
 
 	return __darwintrace_sandbox_check(normPath, flags);
+#undef getattrlist
 #undef readlink
 #undef lstat
 }
