@@ -1,6 +1,5 @@
 /*
  * filemap.c
- * $Id: filemap.c 30972 2007-11-12 17:11:33Z jmpp@macports.org $
  *
  * Copyright (c) 2004 Paul Guyot, The MacPorts Project.
  * All rights reserved.
@@ -30,88 +29,27 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-#if HAVE_CONFIG_H
+#ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <ctype.h>
-#include <errno.h>
-#include <grp.h>
+/* needed for NAME_MAX and PATH_MAX on Linux */
+#define _XOPEN_SOURCE
 
-#if HAVE_STRING_H
-#include <string.h>
-#endif
-
-#if HAVE_STRINGS_H
-#include <strings.h>
-#endif
-
-#if HAVE_DIRENT_H
-#include <dirent.h>
-#endif
-
-#if HAVE_LIMITS_H
-#include <limits.h>
-#endif
-
-#if HAVE_PATHS_H
-#include <paths.h>
-#endif
-
-#ifndef _PATH_DEVNULL
-#define _PATH_DEVNULL   "/dev/null"
-#endif
-
-#include <pwd.h>
-
-#if HAVE_SYS_FILE_H
-#include <sys/file.h>
-#endif
-
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
-#if HAVE_SYS_FCNTL_H
-#include <sys/fcntl.h>
-#endif
-
-#if HAVE_FCNTL_H
-#include <fcntl.h>
-#endif
-
-#if HAVE_SYS_STAT_H
 #include <sys/stat.h>
-#endif
-
-#if HAVE_SYS_WAIT_H
-#include <sys/wait.h>
-#endif
-
-#if HAVE_UNISTD_H
+#include <errno.h>
+#include <fcntl.h>
+#include <limits.h>
 #include <unistd.h>
-#endif
 
 #include <tcl.h>
-#include <tclDecls.h>
 
 #include "filemap.h"
 #include "strcasecmp.h"
 
-/* Avoid a warning with Tcl < 8.4, even if Tcl_GetIndexFromObj's tablePtr
-probably isn't modified. */
-#if (TCL_MAJOR_VERSION > 8) || (TCL_MINOR_VERSION >= 4)
-typedef CONST char* tableEntryString;
-#else
-typedef char* tableEntryString;
-#endif
-
 /* ========================================================================= **
  * Definitions
  * ========================================================================= */
-#pragma mark Definitions
 
 /* ------------------------------------------------------------------------- **
  * Internal structures
@@ -222,6 +160,7 @@ int SetFilemapFromAny(Tcl_Interp* inInterp, Tcl_Obj* inObjPtr);
 int SetResultFromErrorCode(Tcl_Interp* interp, int inErrorCode);
 SFilemapObject* GetObjectFromVarName(Tcl_Interp* interp, Tcl_Obj* inVarName);
 int FilemapCloseCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]);
+int FilemapCreateCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]);
 int FilemapExistsCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]);
 int FilemapGetCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]);
 int FilemapIsReadOnlyCmd(Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[]);
@@ -246,8 +185,6 @@ Tcl_ObjType tclFilemapType = {
 /* ========================================================================= **
  * Tree access functions
  * ========================================================================= */
-#pragma mark -
-#pragma mark Access functions
 
 /**
  * Load the database from a file.
@@ -272,7 +209,7 @@ Load(
 		ssize_t theFileSize;
 
 		/* Open the file for reading, creating it if necessary. */
-		int theFD = open(inDatabasePath, O_RDONLY | O_CREAT, 0664);
+		theFD = open(inDatabasePath, O_RDONLY | O_CREAT, 0664);
 		if (theFD < 0)
 		{
 			theErr = errno;
@@ -374,7 +311,7 @@ Create(
  *
  * @param ioDatabaseBuffer	pointer to the buffer (where the node starts),
  *							updated by this function.
- * @param outTree			on output, a tree in memory.
+ * @param outNode			on output, a tree in memory.
  * @param ioBytesLeft		number of bytes remaining in the buffer (updated
  *							by this function).
  */
@@ -529,8 +466,8 @@ LoadNode(
  * Save the database to the file.
  * This function saves the header and then calls SaveNode.
  *
- * @param inDatabaseFd	file descriptor of the open file (the cursor is reset)
- * @param inTree		tree of the database.
+ * @param inDatabasePath	file descriptor of the open file (the cursor is reset)
+ * @param inTree			tree of the database.
  */
 int
 Save(
@@ -709,6 +646,8 @@ Free(SNode** ioRoot)
 					ckfree((char*) theSubnode);
 					*theSubnodeCursor = 0;
 				}
+				
+				theSubnodeCursor++;
 			}
 		}
 		
@@ -767,7 +706,7 @@ Set(SNode** ioRoot, const char* inPath, const char* inValue)
 
 		/* one char too far */
 		endCursor--;
-		partLength = (int) endCursor - (int) beginCursor;
+		partLength = endCursor - beginCursor;
 
 		/* do we have a node for this entry? */
 		for (indexSubnodes = 0; indexSubnodes < nbSubnodes; indexSubnodes++)
@@ -907,7 +846,7 @@ Get(SNode* inRoot, const char* inPath)
 
 		/* one char too far */
 		endCursor--;
-		partLength = (int) endCursor - (int) beginCursor;
+		partLength = endCursor - beginCursor;
 
 		/* do we have a node for this entry? */
 		for (indexSubnodes = 0; indexSubnodes < nbSubnodes; indexSubnodes++)
@@ -990,7 +929,6 @@ List(SNode* inRoot, const char* inValue)
  * @param outList		the list to populate with paths.
  * @param inSubpath		the path of the current root.
  * @param inSubpathLen	the length, without the terminator, of the path.
- * @return the list of paths which has value for their value.
  */
 void
 ListSubtree(
@@ -1003,7 +941,7 @@ ListSubtree(
 	if (inRoot->fNodeType == kLeaf)
 	{
 		/* it's a leaf. Does the value match? */
-		if (strcmp(((SLeaf*) inRoot)->fValue, inValue) == 0)
+		if (strcasecmp(((SLeaf*) inRoot)->fValue, inValue) == 0)
 		{
 			/* It matches. */
 			char* thePath = ckalloc(inSubpathLen + NAME_MAX + 1);
@@ -1106,7 +1044,7 @@ Delete(SNode** ioRoot, const char* inPath)
 
 		/* one char too far */
 		endCursor--;
-		partLength = (int) endCursor - (int) beginCursor;
+		partLength = endCursor - beginCursor;
 
 		/* do we have a node for this entry? */
 		for (indexSubnodes = 0; indexSubnodes < nbSubnodes; indexSubnodes++)
@@ -1179,8 +1117,6 @@ Delete(SNode** ioRoot, const char* inPath)
 /* ========================================================================= **
  * Tcl object functions
  * ========================================================================= */
-#pragma mark -
-#pragma mark Tcl object functions
 
 /**
  * Free the object.
@@ -1266,8 +1202,6 @@ SetFilemapFromAny(Tcl_Interp* inInterp, Tcl_Obj* inObjPtr UNUSED)
 /* ========================================================================= **
  * Entry points
  * ========================================================================= */
-#pragma mark -
-#pragma mark Entry points
 
 /**
  * Set the result if an error occurred and return TCL_ERROR.
@@ -2021,7 +1955,7 @@ FilemapCmd(
     	kFilemapIsReadOnly
     } EOption;
     
-	static tableEntryString options[] = {
+	static const char *options[] = {
 		"close", "create", "exists", "get", "list", "open", "revert", "save",
 		"set", "unset", "isreadonly", NULL
 	};
@@ -2091,8 +2025,3 @@ FilemapCmd(
 	
 	return theResult;
 }
-
-/* ============================================================== **
-** As of next Thursday, UNIX will be flushed in favor of TOPS-10. **
-** Please update your programs.                                   **
-** ============================================================== */
