@@ -1,8 +1,7 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=tcl:et:sw=4:ts=4:sts=4
 # portbuild.tcl
-# $Id: portbuild.tcl 91557 2012-04-05 02:37:59Z jmr@macports.org $
 #
-# Copyright (c) 2007 - 2012 The MacPorts Project
+# Copyright (c) 2007 - 2013 The MacPorts Project
 # Copyright (c) 2002 - 2004 Apple Inc.
 # All rights reserved.
 #
@@ -43,11 +42,10 @@ namespace eval portbuild {
 }
 
 # define options
-options build.asroot
-options build.jobs
-options build.target
-options build.type
-options use_parallel_build
+options build.asroot \
+        build.jobs \
+        build.target \
+        use_parallel_build
 commands build
 # defaults
 default build.asroot no
@@ -60,51 +58,51 @@ default build.target "all"
 default build.type "default"
 default use_parallel_build yes
 
-# proc to add dependency on bsdmake, if necessary
-option_proc build.type portbuild::set_build_type
-
 set_ui_prefix
 
-proc portbuild::set_build_type {option action args} {
-    array set build_type_map {
-        bsd     {bin:bsdmake:bsdmake}
+# Automatically called from macports1.0 after evaluating the Portfile. If
+# ${build.type} == bsd, ensures bsdmake is present by adding a bin:-style
+# dependency.
+proc portbuild::add_automatic_buildsystem_dependencies {} {
+    global build.type.add_deps
+    if {!${build.type.add_deps}} {
+        return
     }
-
-    if {$action == "set"} {
-        switch $option {
-            build.type {
-                # using [exists foo] doesn't work with arrays!
-                if {[info exists build_type_map([option build.type])]} {
-                    # remove dependencies added before when setting build.type more than once
-                    eval depends_build-delete $build_type_map([option build.type])
-                }
-                if {[info exists build_type_map($args)]} {
-                    # add dependency if needed
-                    eval depends_build-append $build_type_map($args)
-                }
-            }
-        }
+    if {[option build.type] eq "bsd" && [option os.platform] eq "darwin"} {
+        ui_debug "build.type is BSD, adding bin:bsdmake:bsdmake build dependency"
+        depends_build-delete bin:bsdmake:bsdmake
+        depends_build-append bin:bsdmake:bsdmake
+    }
+    if {[option build.type] eq "gnu" && [option os.platform] eq "freebsd"} {
+        ui_debug "build.type is GNU, adding bin:gmake:gmake build dependency"
+        depends_build-delete bin:gmake:gmake
+        depends_build-append bin:gmake:gmake
     }
 }
+# Register the above procedure as a callback after Portfile evaluation
+port::register_callback portbuild::add_automatic_buildsystem_dependencies
+# and an option to turn it off if required
+options build.type.add_deps
+default build.type.add_deps yes
 
 proc portbuild::build_getmaketype {args} {
-    if {[option build.type] == "default"} {
+    if {[option build.type] eq "default"} {
         return [findBinary make $portutil::autoconf::make_path]
     }
     switch -exact -- [option build.type] {
         bsd {
-            if {[option os.platform] == "darwin"} {
+            if {[option os.platform] eq "darwin"} {
                 return [findBinary bsdmake $portutil::autoconf::bsdmake_path]
-            } elseif {[option os.platform] == "freebsd"} {
+            } elseif {[option os.platform] eq "freebsd"} {
                 return [findBinary make $portutil::autoconf::make_path]
             } else {
                 return [findBinary pmake $portutil::autoconf::bsdmake_path]
             }
         }
         gnu {
-            if {[option os.platform] == "darwin"} {
+            if {[option os.platform] eq "darwin"} {
                 return [findBinary gnumake $portutil::autoconf::gnumake_path]
-            } elseif {[option os.platform] == "linux"} {
+            } elseif {[option os.platform] eq "linux"} {
                 return [findBinary make $portutil::autoconf::make_path]
             } else {
                 return [findBinary gmake $portutil::autoconf::gnumake_path]
@@ -112,12 +110,12 @@ proc portbuild::build_getmaketype {args} {
         }
         pbx -
         xcode {
-            if {[option os.platform] != "darwin"} {
+            if {[option os.platform] ne "darwin"} {
                 return -code error "[format [msgcat::mc "This port requires 'xcodebuild', which is not available on %s."] [option os.platform]]"
             }
 
             global xcodebuildcmd
-            if {$xcodebuildcmd != "none"} {
+            if {$xcodebuildcmd ne "none"} {
                 return $xcodebuildcmd
             } else {
                 return -code error "xcodebuild was not found on this system!"
@@ -135,14 +133,20 @@ proc portbuild::build_getjobs {args} {
     set jobs $buildmakejobs
     # if set to '0', use the number of cores for the number of jobs
     if {$jobs == 0} {
-        if {[catch {set jobs [sysctl hw.activecpu]}] || [catch {set memsize [sysctl hw.memsize]}]} {
-            set jobs 2
+        try -pass_signal {
+            set jobs [sysctl hw.activecpu]
+        } catch {{*} eCode eMessage} {
             ui_warn "failed to determine the number of available CPUs (probably not supported on this platform)"
             ui_warn "defaulting to $jobs jobs, consider setting buildmakejobs to a nonzero value in macports.conf"
+            set jobs 2
         }
-        if {[info exists memsize] && $jobs > $memsize / 1000000000 + 1} {
-            set jobs [expr $memsize / 1000000000 + 1]
-        }
+
+        try -pass_signal {
+            set memsize [sysctl hw.memsize]
+            if {$jobs > $memsize / (1024 * 1024 * 1024) + 1} {
+                set jobs [expr {$memsize / (1024 * 1024 * 1024) + 1}]
+            }
+        } catch {*} {}
     }
     if {![string is integer -strict $jobs] || $jobs <= 1} {
         set jobs 1
@@ -151,8 +155,8 @@ proc portbuild::build_getjobs {args} {
 }
 
 proc portbuild::build_getargs {args} {
-    if {(([option build.type] == "default" && [option os.platform] != "freebsd") || \
-         ([option build.type] == "gnu")) \
+    if {(([option build.type] eq "default" && [option os.platform] ne "freebsd") || \
+         ([option build.type] eq "gnu")) \
         && [regexp "^(/\\S+/|)(g|gnu|)make(\\s+.*|)$" [option build.cmd]]} {
         # Print "Entering directory" lines for better log debugging
         return "-w [option build.target]"
