@@ -1,8 +1,7 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=tcl:et:sw=4:ts=4:sts=4
 # portmain.tcl
-# $Id: portmain.tcl 90047 2012-02-20 07:10:07Z jberry@macports.org $
 #
-# Copyright (c) 2004 - 2005, 2007 - 2011 The MacPorts Project
+# Copyright (c) 2004 - 2005, 2007 - 2012 The MacPorts Project
 # Copyright (c) 2002 - 2003 Apple Inc.
 # All rights reserved.
 #
@@ -44,18 +43,20 @@ target_state ${org.macports.main} no
 namespace eval portmain {
 }
 
+set_ui_prefix
+
 # define options
 options prefix name version revision epoch categories maintainers \
         long_description description homepage notes license \
         provides conflicts replaced_by \
         worksrcdir filesdir distname portdbpath libpath distpath sources_conf \
-        os.platform os.subplatform os.version os.major os.arch os.endian \
+        os.platform os.subplatform os.version os.major os.minor os.arch os.endian \
         platforms default_variants install.user install.group \
         macosx_deployment_target universal_variant os.universal_supported \
         supported_archs depends_skip_archcheck installs_libs \
-        copy_log_files \
+        license_noconflict copy_log_files \
         compiler.cpath compiler.library_path \
-        add_users altprefix
+        add_users
 
 # Order of option_proc and option_export matters. Filter before exporting.
 
@@ -65,7 +66,7 @@ option_proc default_variants handle_default_variants
 option_proc notes handle_option_string
 
 # Export options via PortInfo
-options_export name version revision epoch categories maintainers platforms description long_description notes homepage license provides conflicts replaced_by installs_libs
+options_export name version revision epoch categories maintainers platforms description long_description notes homepage license provides conflicts replaced_by installs_libs license_noconflict patchfiles
 
 default subport {[portmain::get_default_subport]}
 proc portmain::get_default_subport {} {
@@ -78,18 +79,17 @@ proc portmain::get_default_subport {} {
 default subbuildpath {[portmain::get_subbuildpath]}
 proc portmain::get_subbuildpath {} {
     global portpath portbuildpath subport
-    if {$subport != ""} {
+    if {$subport ne ""} {
         set subdir $subport
     } else {
         set subdir [file tail $portpath]
     }
-    return [file join $portbuildpath $subdir]
+    return [file normalize [file join $portbuildpath $subdir]]
 }
 default workpath {[getportworkpath_from_buildpath $subbuildpath]}
 default prefix /opt/local
 default applications_dir /Applications/MacPorts
 default frameworks_dir {${prefix}/Library/Frameworks}
-default developer_dir {[portmain::get_developer_dir]}
 default destdir destroot
 default destpath {${workpath}/${destdir}}
 # destroot is provided as a clearer name for the "destpath" variable
@@ -100,8 +100,8 @@ default epoch 0
 default license unknown
 default distname {${name}-${version}}
 default worksrcdir {$distname}
-default filespath {[file join $portpath $filesdir]}
-default worksrcpath {[file join $workpath $worksrcdir]}
+default filespath {[file join $portpath [join $filesdir]]}
+default worksrcpath {[file join $workpath [join $worksrcdir]]}
 # empty list means all archs are supported
 default supported_archs {}
 default depends_skip_archcheck {}
@@ -115,11 +115,12 @@ default install.group {${portutil::autoconf::install_group}}
 default os.platform {$os_platform}
 default os.version {$os_version}
 default os.major {$os_major}
+default os.minor {$os_minor}
 default os.arch {$os_arch}
 default os.endian {$os_endian}
 
 set macosx_version_text {}
-if {[option os.platform] == "darwin"} {
+if {[option os.platform] eq "darwin"} {
     set macosx_version_text "(Mac OS X ${macosx_version}) "
 }
 ui_debug "OS [option os.platform]/[option os.version] ${macosx_version_text}arch [option os.arch]"
@@ -127,7 +128,7 @@ ui_debug "OS [option os.platform]/[option os.version] ${macosx_version_text}arch
 default universal_variant {${use_configure}}
 
 # sub-platforms of darwin
-if {[option os.platform] == "darwin"} {
+if {[option os.platform] eq "darwin"} {
     if {[file isdirectory /System/Library/Frameworks/Carbon.framework]} {
         default os.subplatform macosx
         # we're on Mac OS X and can therefore build universal
@@ -144,47 +145,12 @@ if {[option os.platform] == "darwin"} {
 default compiler.cpath {${prefix}/include}
 default compiler.library_path {${prefix}/lib}
 
-proc portmain::get_developer_dir {} {
-    if {![catch {binaryInPath xcode-select}]
-        && ![catch {exec xcode-select -print-path 2> /dev/null} result]
-        && [file isdirectory $result]} {
-            return $result
-    }
-    global xcodeversion
-    if {[vercmp $xcodeversion 4.3] >= 0} {
-        return "/Applications/Xcode.app/Contents/Developer"
-    } else {
-        return "/Developer"
-    }
-}
-
-# start gsoc08-privileges
-
 # Record initial euid/egid
 set euid [geteuid]
 set egid [getegid]
 
-# if unable to write to workpath, implies running without either root privileges
-# or a shared directory owned by the group so use ~/.macports
-default altprefix {[file join $user_home .macports]}
-if { $euid != 0 && (([info exists workpath] && [file exists $workpath] && ![file writable $workpath]) || ([info exists portdbpath] && ![file writable [file join $portdbpath build]])) } {
-
-    # set global variable indicating to other functions to use ~/.macports as well
-    set usealtworkpath yes
-
-    default worksymlink {[file join ${altprefix}${portpath} work]}
-    default distpath {[file join ${altprefix}${portdbpath} distfiles ${dist_subdir}]}
-    set portbuildpath "${altprefix}${portbuildpath}"
-
-    ui_debug "Going to use alternate build prefix: $altprefix"
-    ui_debug "workpath = $workpath"
-} else {
-    set usealtworkpath no
-    default worksymlink {[file join $portpath work]}
-    default distpath {[file join $portdbpath distfiles ${dist_subdir}]}
-}
-
-# end gsoc08-privileges
+default worksymlink {[file normalize [file join $portpath work]]}
+default distpath {[file normalize [file join $portdbpath distfiles ${dist_subdir}]]}
 
 proc portmain::main {args} {
     return 0
